import fs from 'node:fs';
import path from 'node:path';
import zlib from 'zlib';
import chalk from 'chalk';
import { fileURLToPath } from 'node:url';
import packageJson from '../package.json' with { type: 'json' };

const { green, blue, red, cyan } = chalk;


export function scriptPaths(importMetaUrl) {
  const filename = fileURLToPath(importMetaUrl)
  return {
    __filename: filename,
    __dirname: path.dirname(filename),
  }
}

const { __dirname } = scriptPaths(import.meta.url)

export const useUiPackageJson = () => {
  return packageJson
}

function getSize (code) {
  return (code.length / 1024).toFixed(2) + 'kb'
}

export const createFolder = function (folder) {
  const dir = path.join(__dirname, '..', folder)
  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir)
  }
}

export const writeFile = function (dest, code, zip) {
  const banner = dest.indexOf('.json') > -1
    ? red('[json]')
    : dest.indexOf('.js') > -1
      ? green('[js]  ')
      : dest.indexOf('.ts') > -1
        ? cyan('[ts]  ')
        : blue('[css] ')

  return new Promise((resolve, reject) => {
    function report (extra) {
      console.log(`${banner} ${path.relative(process.cwd(), dest).padEnd(41)} ${getSize(code).padStart(8)}${extra || ''}`)
      resolve(code)
    }

    fs.writeFile(dest, code, err => {
      if (err) return reject(err)
      if (zip) {
        zlib.gzip(code, (err, zipped) => {
          if (err) return reject(err)
          report(` (gzipped: ${getSize(zipped).padStart(8)})`)
        })
      }
      else {
        report()
      }
    })
  })
}

export const readFile = function (file) {
  return fs.readFileSync(file, 'utf-8')
}

export const logError = function (err) {
  console.error('\n' + red('[Error]'), err)
  console.log()
}
